@@ -1,11 +1,159 @@
+package  # hide from cpan
+My::DDCS;
+
+
+# Stolen from Devel::Declare's t/method-no-semi.t
+use Devel::Declare ();
+use Scope::Guard;
+use Sub::Name;
+use strict;
+use warnings;
+
+sub install_keyword {
+    my $class = shift;
+    my $ctx   = $class->new;
+    my %args  = @_;
+    $ctx->{install_cb} = $args{pre_install} || sub {};
+    # I don't really understand why we need to declare method
+    # in the caller's namespace.
+    {
+        no strict 'refs';
+        *{$args{into}.'::'.$args{name}}   = sub (&) {};
+    }
+    Devel::Declare->setup_for(
+        $args{into},
+        { $args{name} => { const => $ctx->mk_parser },
+        },
+    );
+
+}
+
+sub new {
+    my $class = shift;
+    bless {
+        Declarator => undef,
+        Offset     => undef,
+    }, $class;
+}
+
+sub skip_declarator {
+    my $ctx = shift;
+    $ctx->{Offset} += Devel::Declare::toke_move_past_token($ctx->{Offset});
+}
+
+sub skipspace {
+    my $ctx = shift;
+    $ctx->{Offset} += Devel::Declare::toke_skipspace($ctx->{Offset});
+}
+
+sub strip_name {
+    my $ctx = shift;
+    $ctx->skipspace;
+    if (my $len = Devel::Declare::toke_scan_word($ctx->{Offset}, 1)) {
+        my $linestr = Devel::Declare::get_linestr();
+        my $name = substr($linestr, $ctx->{Offset}, $len);
+        substr($linestr, $ctx->{Offset}, $len) = '';
+        Devel::Declare::set_linestr($linestr);
+        return $name;
+    }
+    return;
+}
+
+sub strip_proto {
+    my $ctx = shift;
+    $ctx->skipspace;
+
+    my $linestr = Devel::Declare::get_linestr();
+    if (substr($linestr, $ctx->{Offset}, 1) eq '(') {
+        my $length = Devel::Declare::toke_scan_str($ctx->{Offset});
+        my $proto = Devel::Declare::get_lex_stuff();
+        Devel::Declare::clear_lex_stuff();
+        $linestr = Devel::Declare::get_linestr();
+        substr($linestr, $ctx->{Offset}, $length) = '';
+        Devel::Declare::set_linestr($linestr);
+        return $proto;
+    }
+    return;
+}
+
+sub get_curstash_name {
+    return Devel::Declare::get_curstash_name;
+}
+
+sub shadow {
+    my $ctx = shift;
+    my $pack = $ctx->get_curstash_name;
+    Devel::Declare::shadow_sub("${pack}::$ctx->{Declarator}", $_[0]);
+}
+
+sub inject_if_block {
+    my $ctx = shift;
+    my $inject = shift;
+    $ctx->skipspace;
+    my $linestr = Devel::Declare::get_linestr;
+    if (substr($linestr, $ctx->{Offset}, 1) eq '{') {
+        substr($linestr, $ctx->{Offset}+1, 0) = $inject;
+        Devel::Declare::set_linestr($linestr);
+    }
+}
+
+sub scope_injector_call {
+    my $ctx = shift;
+    return ' BEGIN { ' . __PACKAGE__ . '::inject_scope }; ';
+}
+
+sub parse_proto { }
+sub inject_parsed_proto { }
+sub mk_parser {
+    my $ctx = shift;
+
+    return sub {
+    @{$ctx}{qw(Declarator Offset)} = @_;
+    $ctx->skip_declarator;
+    my $name = $ctx->strip_name;
+    my $proto = $ctx->strip_proto;
+    my @decl = $ctx->parse_proto($proto);
+    my $inject = $ctx->inject_parsed_proto(@decl);
+    if (defined $name) {
+        $inject = $ctx->scope_injector_call().$inject;
+    }
+    $ctx->inject_if_block($inject);
+    if (defined $name) {
+        my $pkg = $ctx->get_curstash_name;
+        $name = join('::', $pkg, $name)
+          unless ($name =~ /::/);
+    }
+    $ctx->shadow(sub (&) {
+        no strict 'refs';
+        my $code = shift;
+        # So caller() gets the subroutine name
+        *{$name} = subname $name => $code if defined $name;
+    });
+    };
+}
+
+sub inject_scope {
+    my $ctx = shift;
+    $^H |= 0x120000;
+    $^H{DD_METHODHANDLERS} = Scope::Guard->new(sub {
+        my $linestr = Devel::Declare::get_linestr;
+        my $offset = Devel::Declare::get_linestr_offset;
+        substr($linestr, $offset, 0) = ';';
+        Devel::Declare::set_linestr($linestr);
+    });
+}
+
+1; # End of My::DDCS
+
 package CGI::Application::Plugin::RunmodeDeclare;
 
 use warnings;
 use strict;
 
-our $VERSION = '0.03_03';
+our $VERSION = '0.01';
 
-use base 'Devel::Declare::MethodInstaller::Simple';
+use base qw/My::DDCS/;
+use Data::Alias ();
 use Carp qw(croak);
 
 sub import {
@@ -16,27 +164,22 @@ sub import {
             runmode   => runmode   =>
             startmode => startmode =>
             errormode => errormode =>
-            invocant  => '$self' =>
-            into      => $caller,
             @_ );
 
-    $class->install_methodhandler(
-        into         => $remap{into},
+    $class->install_keyword(
+        into         => $caller,
         name         => $remap{runmode},
         pre_install  => \&_setup_runmode,
-        invocant     => $remap{invocant},
     );
-    $class->install_methodhandler(
-        into         => $remap{into},
+    $class->install_keyword(
+        into         => $caller,
         name         => $remap{startmode},
         pre_install  => \&_setup_startmode,
-        invocant     => $remap{invocant},
     );
-    $class->install_methodhandler(
-        into         => $remap{into},
+    $class->install_keyword(
+        into         => $caller,
         name         => $remap{errormode},
         pre_install  => \&_setup_errormode,
-        invocant     => $remap{invocant},
     );
 }
 
@@ -44,7 +187,7 @@ sub import {
 my %REGISTRY;
 # per-macro setup
 sub _split {
-    my $n = shift; my ($p,$l) = $n =~ /^(.*?)(?:::(\w*))?$/; return ($p, $l);
+    my $n = shift; my ($p,$l) = $n =~ /(.*)::(.*)/; return ($p, $l);
 }
 sub _setup_runmode {
     my ($fullname, $code) = @_;
@@ -55,15 +198,8 @@ sub _setup_startmode {
     my ($fullname, $code) = @_;
     no strict 'refs'; no warnings 'uninitialized';
     my ($pkg, $name) = _split($fullname);
-    croak "start mode redefined (from $REGISTRY{$pkg}{start_mode_installed})" if $REGISTRY{$pkg}{start_mode_installed};
-    $pkg->add_callback(
-        init => sub {
-            return if exists $_[0]->{__START_MODE_SET_BY_RUNMODEDECLARE};
-            $_[0]->run_modes( [$name] );
-            $_[0]->start_mode($name);
-            $_[0]->{__START_MODE_SET_BY_RUNMODEDECLARE} = 1;
-        }
-    );
+    croak "error mode redefined (from $REGISTRY{$pkg}{error_mode_installed})" if $REGISTRY{$pkg}{error_mode_installed};
+    $pkg->add_callback( init => sub { $_[0]->run_modes([ $name ]); $_[0]->start_mode($name); } );
     $REGISTRY{$pkg}{start_mode_installed} = $fullname;
 }
 sub _setup_errormode {
@@ -71,75 +207,90 @@ sub _setup_errormode {
     no strict 'refs'; no warnings 'uninitialized';
     my ($pkg, $name) = _split($fullname);
     croak "error mode redefined (from $REGISTRY{$pkg}{error_mode_installed})" if $REGISTRY{$pkg}{error_mode_installed};
-    $pkg->add_callback(
-        init => sub {
-            return if exists $_[0]->{__ERROR_MODE_SET_BY_RUNMODEDECLARE};
-            $_[0]->error_mode($name);
-            $_[0]->{__ERROR_MODE_SET_BY_RUNMODEDECLARE} = 1;
-        }
-    );
+    $pkg->add_callback( init => sub { $_[0]->error_mode($name); } );
     $REGISTRY{$pkg}{error_mode_installed} = $fullname;
 }
 
-=begin pod-coverage
-
-=over 4
-
-=item strip_name - we hook into this to install cgiapp callbacks
-
-=item parse_proto - proto parser
-
-=item inject_parsed_proto - turn it into code
-
-=back
-
-=end pod-coverage
-
-=cut
-
 sub strip_name {
     my $ctx = shift;
 
     my $name = $ctx->SUPER::strip_name;
-    $ctx->{pre_install}->($ctx->get_curstash_name . '::' . $name);
+    $ctx->{install_cb}->($ctx->get_curstash_name . '::' . $name);
 
     return $name;
 }
 
 sub parse_proto {
-    my $self = shift;
+    my $ctx = shift;
     my ($proto) = @_;
     $proto ||= '';
 
-    my $invocant = $self->{invocant};
-    $invocant = $1 if $proto =~ s{^(\$\w+):\s*}{};
+    # Do all the signature parsing here
+    my %signature;
+    $signature{invocant} = '$self';
+    $signature{invocant} = $1 if $proto =~ s{^(\$.+):\s*}{};
 
-    my @args = 
-        map { m{^ ([$@%])(\w+) }x ? [$1, $2] : () }
-        split /\s*,\s*/,
-        $proto
-    ;
+    my @protos = split /\s*,\s*/, $proto;
+    for my $idx (0..$#protos) {
+        my $sig = $signature{$idx} = {};
+        my $proto = $protos[$idx];
 
-    return (
-        $invocant,
-        $proto,
-        @args,
-    );
+#            print STDERR "proto: $proto\n";
+
+        $sig->{proto}               = $proto;
+        $sig->{idx}                 = $idx;
+        $sig->{is_at_underscore}    = $proto eq '@_';
+        $sig->{is_ref_alias}        = $proto =~ s{^\\}{}x;
+
+        $sig->{trait}   = $1 if $proto =~ s{ \s+ is \s+ (\S+) \s* }{}x;
+        $sig->{default} = $1 if $proto =~ s{ \s* = \s* (.*) }{}x;
+
+        my($sigil, $name) = $proto =~ m{^ (.)(.*) }x;
+        $sig->{is_optional} = ($name =~ s{\?$}{} or $sig->{default});
+        $sig->{sigil}       = $sigil;
+        $sig->{name}        = $name;
+    }
+
+    # XXX At this point we could do sanity checks
+
+    return \%signature;
 }
 
 # Turn the parsed signature into Perl code
 sub inject_parsed_proto {
-    my $self      = shift;
-    my ($invocant, $proto, @args) = @_;
+    my $ctx = shift;
+    my $signature = shift;
 
     my @code;
-    push @code, "my $invocant = shift;";
-    push @code, "my ($proto) = \@_;" if defined $proto and length $proto;
+    push @code, "my $signature->{invocant} = shift;";
 
-    for my $sig (@args) {
-        my ($sigil, $name) = @$sig;
-        push @code, "${sigil}${name} = ${invocant}->param('${name}') unless defined ${sigil}${name}; ";
-        push @code, "${sigil}${name} = ${invocant}->query->param('${name}') unless defined ${sigil}${name}; ";
+    for( my $idx = 0; my $sig = $signature->{$idx}; $idx++ ) {
+        next if $sig->{is_at_underscore};
+
+        my $sigil = $sig->{sigil};
+        my $name  = $sig->{name};
+
+        # These are the defaults.
+        my $lhs = "my ${sigil}${name}";
+        my $rhs = (!$sig->{is_ref_alias} and $sig->{sigil} =~ /^[@%]$/) ? "\@_[$idx..\$#_]" : "\$_[$idx]";
+
+        # Handle a default value
+        $rhs = "\@_ > $idx ? $rhs : $sig->{default}" if defined $sig->{default};
+
+        # XXX We don't do anything with traits right now
+
+        # XXX is_optional is ignored
+
+        # query params
+        $rhs .= "; ${sigil}${name} = $signature->{invocant}->query->param('${name}') unless defined ${sigil}${name}";
+        $rhs .= "; ${sigil}${name} = $signature->{invocant}->param('${name}') unless defined ${sigil}${name}";
+        # Handle \@foo
+        if( $sig->{is_ref_alias} ) {
+            push @code, sprintf 'Data::Alias::alias(%s = %s);', $lhs, $sigil."{$rhs}";
+        }
+        else {
+            push @code, "$lhs = $rhs;";
+        }
     }
 
     # All on one line.
@@ -158,7 +309,7 @@ CGI::Application::Plugin::RunmodeDeclare - Declare runmodes with keywords
 
 =head1 VERSION
 
-Version 0.03_03
+Version 0.01
 
 =head1 SYNOPSIS
 
@@ -208,12 +359,6 @@ This declares the run mode "foo". Notice how C<$self> is ready for use.
 
 Same as above, only use C<$c> instead of C<$self>.
 
-    use CGI::Application::Plugin::RunmodeDeclare invocant => '$c';
-    runmode baz { $c->quux }
-
-Same as above, but every runmode gets C<$c> by default. You can still say C<runmode ($self:)>
-to rename the invocant.
-
 =item * With a parameter list
 
     runmode baz ( $id, $name ) {
